"""
StateMachineLibrary is robot framework library to create state machine.
"""

from .interface import StateMachineLibrary

__author__ = "marcinooo"
__maintainer__ = "marcinooo"
__version__ = "1.0.1"


__all__ = [
    'StateMachineLibrary',
]
